package fr.nitorac.aurionweb.update;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.PowerManager;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.customview.DialogCustomViewExtKt;
import com.downloader.Error;
import com.downloader.OnDownloadListener;
import com.downloader.PRDownloader;
import com.github.lzyzsd.circleprogress.CircleProgress;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import java.io.File;
import java.lang.ref.WeakReference;

import fr.nitorac.aurionweb.BuildConfig;
import fr.nitorac.aurionweb.R;
import fr.nitorac.aurionweb.Utils;

import static android.content.Context.POWER_SERVICE;

public class UpdateManager {
    public static String CHECK_URL = "https://ranini.iiens.net/aurionweb-check/update.php?version=";

    public static boolean updateAvailable = false;
    public static int downloadId = -1;
    private static UpdateManager instance;

    public UpdateManager(){

    }

    public static void handleOnPause(){
        PRDownloader.pause(downloadId);
    }

    public static void handleOnResume(){
        PRDownloader.resume(downloadId);
    }

    public void checkUpdates(Context c){
        AsyncUpdateCheckerTask task = new AsyncUpdateCheckerTask(c);
        task.execute();
    }

    public void startDownload(final WeakReference<Context> weakContext, String url){
        if(weakContext == null){
            return;
        }
        String tag = "UpdateDownload";
        Context c = weakContext.get();
        final MaterialDialog dialog = new MaterialDialog(c, MaterialDialog.getDEFAULT_BEHAVIOR())
                .title(null, "Téléchargement")
                .icon(null, ContextCompat.getDrawable(c, R.drawable.ic_download))
                .cancelable(false);
        DialogCustomViewExtKt.customView(dialog, R.layout.dialog_download, null, false, false, true, true);
        final WeakReference<CircleProgress> circleProgress = new WeakReference<>(dialog.findViewById(R.id.circle_progress));
        final WeakReference<TextView> textProgress = new WeakReference<>(dialog.findViewById(R.id.progress_text));

        PowerManager powerManager = (PowerManager) weakContext.get().getSystemService(POWER_SERVICE);
        PowerManager.WakeLock wakeLock = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,
                "AurionWeb::DownloadWakelock");

        downloadId = PRDownloader.download(url, c.getCacheDir().getAbsolutePath(), "update.apk")
                .setTag(tag)
                .build()
                .setOnStartOrResumeListener(() -> {
                    if(circleProgress != null){
                        circleProgress.get().setProgress(0);
                        circleProgress.get().setMax(100);
                    }

                    if(textProgress != null){
                        textProgress.get().setText("Chargement ...");
                    }

                    if(!dialog.isShowing()){
                        dialog.show();
                    }

                    if(!wakeLock.isHeld()){
                        wakeLock.acquire(10*60*1000L /*30 minutes*/);
                    }
                })
                .setOnPauseListener(() -> {

                })
                .setOnCancelListener(() -> {
                    if(weakContext != null){
                        Toast.makeText(weakContext.get(), "Téléchargement annulé", Toast.LENGTH_LONG).show();
                    }
                    wakeLock.release();
                })
                .setOnProgressListener(progress -> {
                    int frac = (int)((double)progress.currentBytes * 100.0 / (double)progress.totalBytes);
                    if(circleProgress != null){
                        circleProgress.get().setProgress(frac);
                    }
                    if(textProgress != null){
                        textProgress.get().setText(Utils.bytesToString(progress.currentBytes) + " / " + Utils.bytesToString(progress.totalBytes));
                    }
                })
                .start(new OnDownloadListener() {
                    @Override
                    public void onDownloadComplete() {
                        if(weakContext != null){
                            Context context = weakContext.get();
                            File file = new File(weakContext.get().getCacheDir().getAbsolutePath(), "update.apk");
                            Uri apkURI = FileProvider.getUriForFile(
                                    context,
                                    context.getApplicationContext()
                                            .getPackageName() + ".provider", file);

                            try{
                                Intent intent = new Intent(Intent.ACTION_INSTALL_PACKAGE, apkURI);
                                intent.setDataAndType(apkURI, "application/vnd.android.package-archive");
                                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                                intent.putExtra(Intent.EXTRA_NOT_UNKNOWN_SOURCE, true);
                                dialog.dismiss();
                                throw new Exception();
                                //context.getApplicationContext().startActivity(intent);
                            }catch (Exception e){
                                dialog.dismiss();
                                // Linkify the message
                                final SpannableString s = new SpannableString("Impossible d'installer l'application, veuillez aller sur " + url + " pour installer la mise à jour"); // msg should have url to enable clicking
                                Linkify.addLinks(s, Linkify.ALL);

                                final AlertDialog d = new AlertDialog.Builder(context)
                                        .setPositiveButton(android.R.string.ok, null)
                                        .setIcon(R.drawable.ic_danger)
                                        .setMessage(s)
                                        .create();

                                d.show();

                                // Make the textview clickable. Must be called after show()
                                ((TextView)d.findViewById(android.R.id.message)).setMovementMethod(LinkMovementMethod.getInstance());
                            }
                        }
                        wakeLock.release();
                    }

                    @Override
                    public void onError(Error error) {
                        if(weakContext != null){
                            error.getConnectionException().printStackTrace();
                            Toast.makeText(weakContext.get(), "Erreur : " + error.getConnectionException().getMessage(), Toast.LENGTH_LONG).show();
                        }
                        wakeLock.release();
                    }
                });
    }

    public void spawnUpdateDialog(WeakReference<Context> weakContext, JsonObject remote){
        if(weakContext == null){
            return;
        }
        Context c = weakContext.get();
        StringBuilder changelogs = new StringBuilder();

        try{
            for(JsonElement el : remote.get("changelogs").getAsJsonArray()){
                JsonObject obj = el.getAsJsonObject();
                changelogs.append(obj.get("version").getAsString()).append(" :\n");
                for(JsonElement change : obj.get("msg").getAsJsonArray()){
                    changelogs.append("  - ").append(change.getAsString()).append("\n");
                }
                changelogs.append("\n");
            }

            MaterialDialog dialog = new MaterialDialog(c, MaterialDialog.getDEFAULT_BEHAVIOR())
                    .title(R.string.dialog_update_title, null)
                    .icon(null, ContextCompat.getDrawable(c, R.drawable.ic_download))
                    .message(null, String.format(c.getResources().getString(R.string.dialog_update_msg), BuildConfig.VERSION_NAME, remote.get("version").getAsString(), changelogs), null)
                    .cancelable(true)
                    .positiveButton(null, "METTRE A JOUR", materialDialog -> {
                        startDownload(weakContext, remote.get("url").getAsString());
                        return null;
                    })
                    .negativeButton(null, "ANNULER", materialDialog -> {
                        materialDialog.dismiss();
                        return null;
                    });
            dialog.show();
        }catch (Exception e){
            e.printStackTrace();
            Toast.makeText(weakContext.get(), "Impossible de récupérer les mises à jour (" + e.getLocalizedMessage() + ")", Toast.LENGTH_LONG).show();
        }
    }

    public static UpdateManager getInstance(){
        return instance != null ? instance : (instance = new UpdateManager());
    }
}
